// Design:           Pipeline Accumulator
// Revision:         1.0
// Date:             2013-05-23
// Company:          SCAN
// Designer:         A.Kornukhin (kornukhin@mail.ru)
// License:          GNU GPL 2
// Last modified by: - 
`timescale 1ns/100ps
module sw_accum_pipe
#(
   parameter WIDTH = 16
)
(
   input signed [WIDTH-1:0] din,
   input clk, res,
   output signed [WIDTH-1:0] dout
);
// stack registers for inputs
   reg signed [WIDTH-1:0] pha [WIDTH-1:0];
// stack for outputs
   reg signed [WIDTH-1:0] result [WIDTH-1:0];
   reg [WIDTH-1:0] carry;
   wire [WIDTH-1:0] sum;
   reg [1:0] c [WIDTH-1:0];

   integer j;
   always@(*)
   for(j=0;j<WIDTH;j=j+1)
      c[j][1:0] = result[j][j] + pha[j][j] + (j==0 ? 1'b0 : carry[j-1]);

   genvar i;
   generate
   for(i=0; i<WIDTH; i=i+1)
   begin : accumulator
      assign sum[i] = c[i][0];

      always@(posedge clk or negedge res)
      if(!res)
         carry[i]<=1'b0;
      else
         carry[i]<=c[i][1];

      always@(posedge clk or negedge res)
      if(!res)
         pha[i][WIDTH-1:0]<={WIDTH{1'b0}};
      else
         pha[i][WIDTH-1:0]<=i==0 ? din : pha[i-1];

      always@(posedge clk or negedge res)
      if(!res)
         result[i][WIDTH-1:0]<={WIDTH{1'b0}};
      else
      begin
         if(i==0) 
            result[i][WIDTH-1:0]<={ {(WIDTH-i-1){1'b0}}, sum[i]};
         else if(i==WIDTH-1)
            result[i][WIDTH-1:0]<={ sum[i], result[i-1][i-1:0] };
         else
            result[i][WIDTH-1:0]<={ {(WIDTH-i-1){1'b0}}, sum[i], result[i-1][i-1:0] };
      end

   end
   endgenerate

   assign dout = result[WIDTH-1];

endmodule
