// Design:           Pipeline Accumulator Testbench
// Revision:         1.0
// Date:             2013-05-23
// Company:          SCAN
// Designer:         A.Kornukhin (kornukhin@mail.ru)
// License:          GNU GPL 2
// Last modified by: - 
`timescale 1ns/100ps
//
// Pipelined accumulator testbench
//
module sw_accum_pipe_tb;

   reg clk, res;
   parameter WIDTH = 16;
   reg signed [WIDTH-1:0] pha;
   wire signed [WIDTH-1:0] result;
   integer err_cnt=0;

   sw_accum_pipe
   #(
      .WIDTH(WIDTH)
   ) dut
   (
      .din(pha),
      .clk(clk),
      .res(res),
      .dout(result)
   );

   always #5 clk=!clk;
   initial
   begin
      #0 res=0; clk=0; pha=0;
      #3 res=1;
      #10 pha=2;//2
      #10 pha=7;//9
      #10 pha=12;//21
      #10 pha=-1;//20
      #10 pha=-1;//19
      #10 pha=-2;//17
      #10 pha=6;//23
      #10 pha=4;//27
      #10 pha=420;//447
      #10 pha=-571;//-124
      #10 pha=31;//-93
      #10 pha=42;//-51
      #10 pha=16;//-35
      #10 pha=178;//143
      #10 pha=-23;//120
      #10 pha=32760;//32760 + 120 = 32880 -> wrap: 32880 - 32768 = 112 -> MIN + 112 = -32768 + 112 = -32656
      #10 pha = 32000;//-656
      #10 pha = 655;//-1
      #10 pha = 2;//1
      #10 pha=0;
   end

  initial
  begin
  $dumpfile("1.vcd");
  $dumpvars(10);
      #(WIDTH*10+13);
      #10 if(result !== 2) begin err_cnt=err_cnt+1; $display("   Error: result 1 equal to %d (expected 2).", result); end
      #10 if(result !== 9) begin err_cnt=err_cnt+1; $display("   Error: result 2 equal to %d (expected 9).", result); end
      #10 if(result !== 21) begin err_cnt=err_cnt+1; $display("   Error: result 3 equal to %d (expected 21).", result); end
      #10 if(result !== 20) begin err_cnt=err_cnt+1; $display("   Error: result 4 equal to %d (expected 20).", result); end
      #10 if(result !== 19) begin err_cnt=err_cnt+1; $display("   Error: result 5 equal to %d (expected 19).", result); end
      #10 if(result !== 17) begin err_cnt=err_cnt+1; $display("   Error: result 6 equal to %d (expected 17).", result); end
      #10 if(result !== 23) begin err_cnt=err_cnt+1; $display("   Error: result 7 equal to %d (expected 23).", result); end
      #10 if(result !== 27) begin err_cnt=err_cnt+1; $display("   Error: result 8 equal to %d (expected 27).", result); end
      #10 if(result !== 447) begin err_cnt=err_cnt+1; $display("   Error: result 9 equal to %d (expected 447).", result); end
      #10 if(result !== -124) begin err_cnt=err_cnt+1; $display("   Error: result 10 equal to %d (expected -124).", result); end
      #10 if(result !== -93) begin err_cnt=err_cnt+1; $display("   Error: result 11 equal to %d (expected -93).", result); end
      #10 if(result !== -51) begin err_cnt=err_cnt+1; $display("   Error: result 12 equal to %d (expected -51).", result); end
      #10 if(result !== -35) begin err_cnt=err_cnt+1; $display("   Error: result 13 equal to %d (expected -35).", result); end
      #10 if(result !== 143) begin err_cnt=err_cnt+1; $display("   Error: result 14 equal to %d (expected 143).", result); end
      #10 if(result !== 120) begin err_cnt=err_cnt+1; $display("   Error: result 15 equal to %d (expected 120).", result); end
      #10 if(result !== -32656) begin err_cnt=err_cnt+1; $display("   Error: result 16 equal to %d (expected -32656).", result); end
      #10 if(result !== -656) begin err_cnt=err_cnt+1; $display("   Error: result 17 equal to %d (expected -656).", result); end
      #10 if(result !== -1) begin err_cnt=err_cnt+1; $display("   Error: result 18 equal to %d (expected -1).", result); end
      #10 if(result !== 1) begin err_cnt=err_cnt+1; $display("   Error: result 19 equal to %d (expected 1).", result); end
      #10 $display("\n\n   Simulation of 'sw_accum_pipe' (%0d) completed with %0d errors.\n\n", WIDTH, err_cnt);
      #100 $finish;
   end

endmodule
